// original source http://mgeshaderlibrary.wikispaces.com/SSNM
// Author: peachykeen
// Title: SSNM v2
// converted by Ren712
// make_normals.fx
//


//---------------------------------------------------------------------
// Settings
//---------------------------------------------------------------------
float2 rcpres = float2(0.0015,0.0015);
texture sTex0 :TEX0;

//---------------------------------------------------------------------
// Include some common stuff
//---------------------------------------------------------------------
#include "mta-helper.fx"
texture gDepthBuffer : DEPTHBUFFER;

//---------------------------------------------------------------------
// Sampler for the main texture
//---------------------------------------------------------------------
sampler Sampler0 = sampler_state
{
    Texture = (sTex0);
	minfilter = linear; 
	magfilter = linear; 
	mipfilter = linear; 
	addressu = clamp; 
	addressv = clamp;
};

sampler sDepth = sampler_state 
{ 
	texture = (gDepthBuffer); 
	minfilter = linear; 
	magfilter = linear; 
	mipfilter = linear; 
	addressu = clamp; 
	addressv = clamp;
};

//---------------------------------------------------------------------
// Structure of data sent to the pixel shader ( from the vertex shader )
//---------------------------------------------------------------------
struct PSInput
{
  float4 Position : POSITION0;
  float4 Diffuse : COLOR0;
  float2 TexCoord : TEXCOORD0;
};

float3 GetVSPosition(float2 UV)
{
       // these should go in pre-shaders
       //
       static const float FOV = 75.0;
   static float2 aratio = float2(rcpres.y/rcpres.x, 1.0);
   static const float nearPlaneDist = 1.0/tan(radians(FOV*0.5));

       // calculate the view-space position from screen UV coords
       //
   float3 dir = normalize(float3((UV*2.0-1.0)*aratio, nearPlaneDist));
   float depth = tex2D(sDepth , UV).r;
   return depth*dir;
}


//------------------------------------------------------------------------------------------
// PixelShaderFunction
//  1. Read from PS structure
//  2. Process
//  3. Return pixel color
//------------------------------------------------------------------------------------------
float4 PixelShaderFunction(PSInput PS) : COLOR0
{
       // calculate the view-space positions of the pixel and some
       // of the surrounding pixels
       //
       float4 oCol = 0;
   float3 pos = GetVSPosition(PS.TexCoord);
   float3 posxp1 = GetVSPosition(PS.TexCoord + float2(rcpres.x,0.0));
   float3 posyp1 = GetVSPosition(PS.TexCoord + float2(0.0,rcpres.y));
   
       // use a forward-difference approximation to the partial derivatives
       //
   float dxdz = (posxp1.z - pos.z)/distance(posxp1.x, pos.x);
   float dydz = (posyp1.z - pos.z)/distance(posyp1.y, pos.y);

       // use the partial derivatives to construct the normal and pack it in
       // to the render target
       //
   float3 normal = normalize(float3(-dxdz, -dydz, 1.0));
   oCol = float4(normal*0.5+0.5, 1.0);

	float4 nMask = tex2D(Sampler0, PS.TexCoord) - 0.5;

	//normal.rb *= nShift;
	normal += nMask;

	oCol = float4(normal.rgb, 1);
	return oCol;
}


//------------------------------------------------------------------------------------------
// Techniques
//------------------------------------------------------------------------------------------
technique tec0
{
    pass P0
    {
        PixelShader = compile ps_2_0 PixelShaderFunction();
    }
}

// Fallback
technique fallback
{
    pass P0
    {
        // Just draw normally
    }
}
